﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;
using Bricksoft;
using Bricksoft.IM.ICQ;

namespace Bricksoft.IM.ICQ.Demo
{
    public partial class frmRobot : Form
    {
        private ICQClient im = new ICQClient();
        private TreeNode rootNode = null;

        public delegate void MyInvoke(string str);

        public void showLog(string log)
        {
            if (txtLog.InvokeRequired)
            {
                MyInvoke _myInvoke = new MyInvoke(showLog);
                this.Invoke(_myInvoke, new object[] { log });
            }
            else
            {
                txtLog.Text = "[" + System.DateTime.Now.ToLocalTime() + "] " + log + "\r\n" + txtLog.Text;
            }
        }

        public frmRobot()
        {
            InitializeComponent();
          
            cbStatus.Items.Add("Online");
            cbStatus.Items.Add("Away");
            cbStatus.Items.Add("N/A");
            cbStatus.Items.Add("Occupied");

            im.OnError += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            im.OnLog += delegate(object sender, string log) { this.Invoke(new OnLogEventHandler(OnLog), new object[] { sender, log }); };
            im.OnLogin += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogin)); };
            im.OnLogout += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnLogout)); };
            im.OnReceiveBuddyList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveBuddyList)); };
            im.OnReceiveGroupList += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnReceiveGroupList)); };
            im.OnBuddyStatusChanged += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnBuddyStatusChanged), new object[] { sender, buddy }); };
            im.OnMessage += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessage), new object[] { sender, buddy, message }); };
            im.OnMessageSent += delegate(object sender, string buddy, string message) { this.Invoke(new OnMessageEventHandler(OnMessageSent), new object[] { sender, buddy, message }); };
            im.OnStatusChanged += delegate(object sender, EventArgs e) { this.Invoke(new EventHandler(OnStatusChanged), new object[] { sender, e }); };
            im.OnDeleteBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnDeleteBuddy), new object[] { sender, buddy }); };
            im.OnAddBuddy += delegate(object sender, string buddy) { this.Invoke(new OnBuddyEventHandle(OnAddBuddy), new object[] { sender, buddy }); };
            im.OnBuddyAddYouRequest += delegate(object sender, string buddy, ref bool bAccept) {
                object[] inVar = { sender, buddy, bAccept };
                this.Invoke(new OnBuddyAddYouRequestEventHandler(OnBuddyAddYouRequest), inVar);
                bAccept = (bool)inVar[2];
            };

            im.OnNicknameChanged += delegate(object sender, EventArgs e){ this.Invoke(new EventHandler(OnNicknameChanged));};
        }

        private void OnBuddyAddYouRequest(object sender, string buddy, ref bool accept)
        {
            if (MessageBox.Show("\""+ buddy + "\" added you. Do you want to \"" + buddy + "\" to see you?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                accept = true;
            }
            else
                accept = false;
        }

        private void OnAddBuddy(object sender, string buddy)
        {
            if (im.BuddyList.ContainsKey(buddy))
            {
                ICQBuddy bo = (ICQBuddy)im.BuddyList[buddy];

                if (im.GroupList.ContainsKey(bo.GroupID))
                {
                    OSCARGroup go = (OSCARGroup)im.GroupList[bo.GroupID];

                    TreeNode bn = new TreeNode(bo.Account);
                    bn.ImageKey = statusAsImageKey(bo.Status);
                    bn.SelectedImageKey = bn.ImageKey;
                    bn.Tag = bo;
                    bo.Tag = bn;

                    ((TreeNode)go.Tag).Nodes.Add(bn);
                    ((TreeNode)go.Tag).ExpandAll();
                }
            }
        }

        private void OnDeleteBuddy(object sender, string buddy)
        {
            ((TreeNode)((ICQBuddy)im.BuddyList[buddy]).Tag).Remove();
        }

        private void OnStatusChanged(object sender, EventArgs e)
        {
            rootNode.ImageKey = statusAsImageKey(im.Status);
            rootNode.SelectedImageKey = rootNode.ImageKey;
            /*
            cbStatus.Items.Add("Online");
            cbStatus.Items.Add("Away");
            cbStatus.Items.Add("N/A");
            cbStatus.Items.Add("Occupied");
            cbStatus.Items.Add("Disconnect");
            */
            switch (im.Status)
            {
                case ICQStatus.icqNA:
                    cbStatus.SelectedIndex = 2;
                    break;
                case ICQStatus.icqAway:
                    cbStatus.SelectedIndex = 1;
                    break;
                case ICQStatus.icqOccupied:
                    cbStatus.SelectedIndex = 3;
                    break;
                default:
                    cbStatus.SelectedIndex = 4;
                    break;
                case ICQStatus.icqOnline:
                    cbStatus.SelectedIndex = 0;
                    break;
            }
        }

        private string statusAsImageKey(ICQStatus s)
        {
            switch (s)
            {
                case ICQStatus.icqNA:
                    return "na.png";
                case ICQStatus.icqAway:
                    return "away.png";
                case ICQStatus.icqOccupied:
                    return "occupied.png";
                default:
                    return "disconnect.png";
                case ICQStatus.icqOnline:
                    return "online.png";
            }

        }

        private void OnMessage(object sender, string buddy, string message)
        {
            showLog(buddy + " says: " + message);            
        }

        private void OnMessageSent(object sender, string buddy, string message)
        {
            showLog("Send \"" + message + "\" to " + buddy);
        }

        private void OnBuddyStatusChanged(object sender, string buddy)
        {
            ICQBuddy b = (ICQBuddy)im.BuddyList[buddy];
            ((TreeNode)b.Tag).ImageKey = statusAsImageKey(b.Status);
            ((TreeNode)b.Tag).SelectedImageKey = ((TreeNode)b.Tag).ImageKey;
        }

        private void OnLogin(object sender, EventArgs e)
        {
            rootNode = new TreeNode(im.Account);
            rootNode.ImageKey = statusAsImageKey(im.Status);
            rootNode.SelectedImageKey = rootNode.ImageKey;

            cmdLogin.Enabled = false;
            cbStatus.Enabled = true;

            tvIM.Nodes.Add(rootNode);

            rootNode.Text = im.Account;

            tabControl1.SelectedIndex = 1;

            lblStatus.Text = "Login";
        }

        private void OnLogout(object sender, EventArgs e)
        {
            tvIM.Nodes.Clear();

            cmdLogin.Enabled = true;
            cbStatus.Enabled = false;

            tabControl1.SelectedIndex = 0;

            lblStatus.Text = "Logout.";
        }

        private void OnNicknameChanged(object sender, EventArgs e)
        {
            if (im.Nickname != "")
                rootNode.Text = im.Nickname;
            else
                rootNode.Text = im.Account;
        }

        private void OnLog(object sender, string log)
        {
            showLog(log);
            lblStatus.Text = log;
        }

        private TreeNode FindNode(TreeNode tnParent, int v)
        {

            if (tnParent == null) return null;

            if ((tnParent.ImageKey == "group.png") && ((((OSCARGroup)tnParent.Tag).GroupID == v)))
                return tnParent;

            TreeNode tnRet = null;

            foreach (TreeNode tn in tnParent.Nodes)
            {
                tnRet = FindNode(tn, v);
                if (tnRet != null) break;
            }

            return tnRet;
        }

        private void OnReceiveBuddyList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry buddy in im.BuddyList)
            {
                ICQBuddy b = (ICQBuddy)buddy.Value;

                TreeNode bn = new TreeNode(b.Account);

                TreeNode gn = FindNode(rootNode, b.GroupID);

                if (gn != null)
                {
                    bool bExist = false;

                    for (int k = 0; k < gn.Nodes.Count; k++)
                        if (b.Account == ((ICQBuddy)gn.Nodes[k].Tag).Account)
                        {
                            bExist = true;

                            break;
                        }

                    if (!bExist)
                    {
                        if (b.Nickname != "")
                            bn.Text = b.Nickname;
                        else
                            bn.Text = b.Account;

                        bn.ImageKey = statusAsImageKey(b.Status);
                        bn.SelectedImageKey = bn.ImageKey;
                        bn.Tag = b;
                        b.Tag = bn;

                        gn.Nodes.Add(bn);

                        gn.Text = ((OSCARGroup)gn.Tag).GroupName + " (" + gn.Nodes.Count + ")";
                    }
                }
                
            }

            rootNode.ExpandAll();
        }

        private void OnReceiveGroupList(object sender, EventArgs e)
        {
            foreach (DictionaryEntry group in im.GroupList)
            {
                OSCARGroup g = (OSCARGroup)group.Value;

                bool bExist = false;

                for(int i=0;i<rootNode.Nodes.Count;i++)
                    if (g.GroupID == ((OSCARGroup)rootNode.Nodes[i].Tag).GroupID)
                    {
                        bExist = true;

                        break;
                    }

                if (!bExist)
                {
                    TreeNode gn = new TreeNode(g.GroupName);
                    rootNode.Nodes.Add(gn);
                    gn.ImageKey = "group.png";
                    gn.SelectedImageKey = gn.ImageKey;
                    gn.Tag = g;
                    g.Tag = gn;
                }
            }

            rootNode.ExpandAll();
        }

        private void cmdLogin_Click(object sender, EventArgs e)
        {
            im.Account = txtEmail.Text;
            im.Password = txtPassword.Text;
            im.Login();
        }

        private void cbStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            /*
            cbStatus.Items.Add("Online");
            cbStatus.Items.Add("Away");
            cbStatus.Items.Add("N/A");
            cbStatus.Items.Add("Occupied");
            cbStatus.Items.Add("Disconnect");
             */
            switch (cbStatus.SelectedIndex)
            {
                case 0:
                    im.Status = ICQStatus.icqOnline;
                    break;
                case 1:
                    im.Status = ICQStatus.icqAway;
                    break;
                case 2:
                    im.Status = ICQStatus.icqNA;
                    break;
                case 3:
                    im.Status = ICQStatus.icqOccupied;
                    break;
                case 4:
                    im.Status = ICQStatus.icqOffline;
                    break;
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null) && (tvIM.SelectedNode.Tag is ICQBuddy))
            {
                im.DeleteBuddy((tvIM.SelectedNode.Tag as ICQBuddy).Account);
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void addBuddyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //add buddy
            if (im.Logined)
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Add Buddy:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                    im.AddBuddy(frmInput.MessageStr);
            }
        }

        private void logoutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            im.Logout();
        }


        private void sendMessageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((tvIM.SelectedNode != null)&&(tvIM.SelectedNode.Tag is ICQBuddy))
            {
                InputMessageForm frmInput = new InputMessageForm();
                frmInput.setCaption("Instant Message:");
                frmInput.ShowDialog();

                if ((frmInput.Result == DialogResult.OK) && (frmInput.MessageStr != ""))
                {
                    im.SendMessage((tvIM.SelectedNode.Tag as ICQBuddy).Account, frmInput.MessageStr);
                }
            }
        }

        private void loginToolStripMenuItem_Click(object sender, EventArgs e)
        {
            cmdLogin_Click(sender,e);
        }

        private void frmRobot_Shown(object sender, EventArgs e)
        {
        }

        private void frmRobot_Activated(object sender, EventArgs e)
        {
        }

        private void frmRobot_FormClosed(object sender, FormClosedEventArgs e)
        {
            im.Logout();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            im.Nickname = "bstest111";
        }

    }
}